import plotly.graph_objects as go
from amplabs.variables import *
import amplabs.variables as avs
import random


def update_line_chart(x_axes, y_axes_values):
    fig = go.Figure()
    # Create y-axes for the selected traces
    for i, y_values in enumerate(y_axes_values):
        if len(y_values) == 0 or x_axes[i] == "":
            continue
        axis_num = i + 1
        df = data_list[i]
        if len(color_list[i]) == 0:
            color_list[i] = colorTransitions[i]
        if i == 0:
            for j, y_axis in enumerate(y_values):
                if j >= len(color_list[i]):
                    color = "#{:06x}".format(random.randint(0, 0xFFFFFF))
                    color_list[i].append(color)
                else:
                    color = color_list[i][j]

                y_df = [float(temp) for temp in df[y_axis]]
                x_df = [float(temp) for temp in df[x_axes[i]]]
                fig.add_trace(
                    go.Scatter(
                        x=x_df,
                        y=y_df,
                        name=y_axis,
                        line=dict(color=color, width=line_width[i]),
                        showlegend=True,
                    )
                )
            fig.update_layout(
                yaxis=dict(
                    title=ylabel[i],
                    titlefont=dict(color="#ff7f0e"),
                    tickfont=dict(color="#ff7f0e"),
                    ticks="outside",
                )
            )
        else:
            for j, y_axis in enumerate(y_values):
                if j >= len(color_list[i]):
                    color = "#{:06x}".format(random.randint(0, 0xFFFFFF))
                    color_list[i].append(color)
                else:
                    color = color_list[i][j]
                y_df = [float(temp) for temp in df[y_axis]]
                x_df = [float(temp) for temp in df[x_axes[i]]]
                fig.add_trace(
                    go.Scatter(
                        x=x_df,
                        y=y_df,
                        name=y_axis,
                        yaxis=f"y{axis_num}",
                        line=dict(color=color, width=line_width[i]),
                    )
                )
                fig.update_layout(
                    **{
                        f"yaxis{axis_num}": dict(
                            title=ylabel[i],
                            overlaying="y",
                            side="right",
                            titlefont=dict(color="#ff7f0e"),
                            tickfont=dict(color="#ff7f0e"),
                            autoshift=True,
                            anchor="free",
                            ticks="outside",
                            shift=20 * (i - 1),
                        )
                    }
                )
    fig.update_layout(
        dict(
            legend={"x": 1.05, "y": 0.9},
        ),
        width=1200,
        height=600,
        xaxis_title=avs.xlabel,
    )

    for i, x_range in enumerate(
        x_ranges,
    ):
        if len(x_range) == 0:
            continue
        x_axis_name = "xaxis"  # Generate unique x-axis identifier
        fig.update_layout(**{x_axis_name: dict(range=x_range)})

    for i, y_range in enumerate(
        y_ranges,
    ):
        if len(y_range) == 0:
            continue
        y_axis_name = "yaxis"
        y_num = i + 1
        if y_num != 1:
            y_axis_name = y_axis_name + str(y_num)
        fig.update_layout(
            **{
                y_axis_name: dict(
                    range=y_range,
                )
            }
        )

    for i, y_tick in enumerate(
        y_interval,
    ):
        if isinstance(y_tick, (int, float)) is False:
            continue
        y_axis_name = "yaxis"
        y_num = i + 1
        tick_num = y_tick
        if y_num != 1:
            y_axis_name = y_axis_name + str(y_num)
        fig.update_layout(
            **{
                y_axis_name: dict(
                    dtick=tick_num,
                )
            }
        )

    for i, x_tick in enumerate(
        x_interval,
    ):
        if isinstance(x_tick, (int, float)) is False:
            continue
        x_axis_name = "xaxis"
        fig.update_layout(
            **{
                x_axis_name: dict(
                    dtick=x_tick,
                )
            }
        )

    # Update layout with dynamically generated x-axes
    fig.update_xaxes(
        showline=True,
        linewidth=1,
        linecolor="black",
        mirror=True,
        ticks="outside",
    )
    fig.update_yaxes(
        showline=True,
        linewidth=1,
        linecolor="black",
        mirror=True,
        # ticks="outside",
    )
    return fig


from dash import html
from amplabs.components import navbar, graph, selectionBar
from amplabs.variables import *

def setup_layout():
    return html.Div(
        [
            navbar.HTML_NAVBAR,
            selectionBar.htmlSelectionBar(data_names, list_of_headers),
            graph.HTML_GRAPH,
        ],
        style={"fontSize": "14px"},
    )


import pandas as pd
import numpy as np



def minutesToSeconds(column):
    column = pd.to_numeric(column) / 60
    return column



def showHeaders(df):
    headers = []
    for col in df.columns:                                               
        headers.append(col)
    return headers      


def convertToExcel(df, file_path):
    df.to_excel(file_path, index=False, engine='openpyxl')



def addHeaders(df, column_list):
    df.columns = column_list
    return df



def addTestTime(df, start_time, increment):
    num_rows = len(df)                                 # Number of rows present in the dataset

    df['Test Time'] = np.linspace(start_time, start_time + (num_rows - 1) * increment, num_rows)
    return df


def dataSmoothing(column):
    window_size = 800  
    column = column.rolling(window=window_size).mean().dropna()
    return column


from amplabs.reader import *
from amplabs.plot import *
from amplabs.utils import *
import os


current_directory = os.path.dirname(os.path.abspath(__file__))

relative_dat_path = os.path.join(current_directory, 'files','DATTest.dat')
relative_tdms_path = os.path.join(current_directory, 'files','TDMSTest.tdms')
relative_csv_path = os.path.join(current_directory, 'files','CSVTest.csv')


''' Test for reader '''

dat_data = readDAT(relative_dat_path)
tdms_data = readTDMS(relative_tdms_path)
csv_data = readCSV(relative_csv_path)

print("Reader methods working...")


''' Test for utils '''

dat_data["Test Time"] = minutesToSeconds(dat_data["Test Time"])
headers = showHeaders(dat_data)
newtdms = addHeaders(tdms_data, ['Amphenol', 'LT_Sens2', 'Voltage', 'WaterFlow', 'AbsoluteHumidity BSS_V7', 'AirTemperature BSS_V7', 'Ethanol BSS_V7', 'TotalVOC BSS_V7', 'H2_Concentration AX222058_V0', 'RelativeHumidity AX222058_V0'])
tdms_data = addTestTime(tdms_data, 0, 0.1)
csv_data["voltage"] = dataSmoothing(csv_data["voltage"])

print("Utils methods working...")


# ''' Test for plots - dashboard '''

# ax1 = plot(df=dat_data,df_name="AnD_data")

# ax1.set_color(["#1f77b4", "#ff7f0e", "#2ca02c", "#d62728", "#9467bd"])
# ax1.set_xrange([0,100])
# ax1.set_yrange([10,90])
# ax1.set_xintervals(10)
# ax1.set_yintervals(10)
# ax1.set_xlabel("X test")
# ax1.set_ylabel("Y test")
# ax1.set_linewidth(3.5)

# plot.plot_on_dashboard()


''' Test for plots - svg ''' 

ax2 = plot(df=tdms_data, x_columns="Test Time", y_columns=["Voltage","Amphenol"])
ax2.set_color(["#1f77b4", "#ff7f0e", "#2ca02c", "#d62728", "#9467bd"])
ax2.set_xrange([0,100])
ax2.set_yrange([10,90])
ax2.set_xintervals(10)
ax2.set_yintervals(10)
ax2.set_xlabel("X test")
ax2.set_ylabel("Y test")
ax2.set_linewidth(3.5)

plot.plot_as_svg()

print("Plots method working...")


import pandas as pd
import re
from nptdms import TdmsFile
import numpy as np



def readCSV(file_path):
    df = pd.read_csv(file_path)
    return df



def readDAT(file_path):
    try:
        dat_data = []

        regex_patterns = [                                                          # Define the regex patterns and replacements
            (re.compile(r"^(?!1).*.[\r\n|\n]", re.MULTILINE), ""),                  # Remove lines not starting with '1'
            (re.compile(r"^(1\t*ALIAS\t)", re.MULTILINE), ""),                      # Remove lines starting with '1 ALIAS'
            (re.compile(r"^(1.*DATA\t)", re.MULTILINE), ""),                        # Remove lines starting with '1 DATA'
            (re.compile(r"^(1.*PARAMS).*.[\r\n|\n]", re.MULTILINE), ""),            # Remove lines starting with '1 PARAMS'
            (re.compile(r"^(1.*UNITS).*.[\r\n|\n]", re.MULTILINE), ""),             # Remove lines starting with '1 UNITS'
            (re.compile(r"\t+", re.MULTILINE), ","),                                # Replace tabs with commas
        ]

        with open(file_path, 'r', encoding='ISO-8859-1') as file:               # Load the data from the file
            for line in file:
                if not line.startswith("#"):                                        # Apply regex replacements
                    for pattern, replacement in regex_patterns:
                        line = pattern.sub(replacement, line)
                    if line:
                        dat_data.append(line.strip())

        df = pd.DataFrame({'Data': dat_data})                                       # Create a DataFrame from the extracted data
        df = df['Data'].str.split(',',expand=True)
        df.columns = df.iloc[0]                                                     # Adjust the headers of the data
        df = df[1:]
        df.reset_index(drop=True, inplace=True)

        return df       
    except Exception as e:
        print(f"Error: {e}")



def readTDMS(file_path):
    try:
        tdms_file = TdmsFile.read(file_path, True)                                 # Read TDMS file
        df = tdms_file.as_dataframe(True)
        channel_data_dict = {}
        max_length = 0

        for group in tdms_file.groups():                                         # Get group, channel and channel data from the file
            for channel in group.channels():
                channel_data = channel[:]
                if len(channel_data) > max_length:
                    max_length = len(channel_data)
                channel_data_dict[channel.name] = channel_data

        for channel_name, channel_data in channel_data_dict.items():             # Pad the channel data with NaN to match the maximum length
            if len(channel_data) < max_length:
                channel_data_dict[channel_name] = np.concatenate((channel_data, [np.nan] * (max_length - len(channel_data))))

        df = pd.DataFrame(channel_data_dict)                                     # Create a dataframe from the channel data
        df.columns = [col.replace('\n', ' ') for col in df.columns]

        return df                                                                # Return the final dataframe
    except Exception as e:
        print(f"Error: {e}")


from dash import dcc
import dash_bootstrap_components as dbc


HTML_GRAPH = dbc.Row(
    [
        dbc.Col(
            dcc.Graph(
                id="graph", style={"display": "grid", "justifyContent": "center"}
            ),
        )
    ]
)
